@@ -62,7 +62,7 @@ struct latencyStats {
 };
 
 void latencyMonitorInit(void);
-void latencyAddSample(char *event, mstime_t latency);
+void latencyAddSample(const char *event, mstime_t latency);
 int THPIsEnabled(void);
 
 /* Latency monitoring macros. */
